package vn.hus.nlp.fsm.builder;

import java.util.Arrays;
import java.util.List;
import vn.hus.nlp.fsm.FSM;
import vn.hus.nlp.fsm.ISimulator;
import vn.hus.nlp.fsm.fsa.DFA;
import vn.hus.nlp.fsm.fst.FST;
import vn.hus.nlp.fsm.io.FSMMarshaller;
import vn.hus.nlp.fsm.util.FSMUtilities;
import vn.hus.nlp.utils.UTF8FileUtility;

public abstract class FSMBuilder
{
  protected FSM machine;

  public FSMBuilder(String paramString)
  {
    if (paramString.equalsIgnoreCase("DFA"))
      this.machine = new DFA();
    if (paramString.equalsIgnoreCase("FST"))
      this.machine = new FST();
  }

  protected abstract void addItem(String paramString, String[] paramArrayOfString);

  public void create(String[] paramArrayOfString, String[][] paramArrayOfString1)
  {
    for (int i = 0; i < paramArrayOfString.length; i++)
      addItem(paramArrayOfString[i], paramArrayOfString1[i]);
    finalize();
  }

  public void create(String[] paramArrayOfString)
  {
    Arrays.sort(paramArrayOfString, 0, paramArrayOfString.length);
    String[][] arrayOfString = new String[paramArrayOfString.length][];
    create(paramArrayOfString, arrayOfString);
  }

  public void create(List<String> paramList)
  {
    create((String[])paramList.toArray(new String[paramList.size()]));
  }

  public void create(List<String> paramList, List<String[]> paramList1)
  {
    create((String[])paramList.toArray(new String[paramList.size()]), (String[][])paramList1.toArray(new String[paramList1.size()][]));
  }

  public void create(String paramString)
  {
    String[] arrayOfString = UTF8FileUtility.getLines(paramString);
    Arrays.sort(arrayOfString, 0, arrayOfString.length);
    create(arrayOfString);
  }

  protected abstract void finalize();

  public void encode(String paramString)
  {
    new FSMMarshaller().marshal(this.machine, paramString);
    FSMUtilities.statistic(this.machine);
  }

  public void printMachine()
  {
    new FSMMarshaller().marshal(this.machine, System.out);
    FSMUtilities.statistic(this.machine);
  }

  public FSM getMachine()
  {
    return this.machine;
  }

  public ISimulator getSimulator()
  {
    return getMachine().getSimulator();
  }

  public void dispose()
  {
    getMachine().dispose();
  }
}

/* Location:           C:\Downloads\vn.hus.nlp.tokenizer-4.1.1-bin.tar\lib\vn.hus.nlp.fsm-1.0.0.jar
 * Qualified Name:     vn.hus.nlp.fsm.builder.FSMBuilder
 * JD-Core Version:    0.6.0
 */