package vn.hus.nlp.fsm.builder;

import java.io.PrintStream;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import vn.hus.nlp.fsm.Configuration;
import vn.hus.nlp.fsm.FSM;
import vn.hus.nlp.fsm.ISimulator;
import vn.hus.nlp.fsm.State;
import vn.hus.nlp.fsm.Transition;
import vn.hus.nlp.fsm.fsa.DFAConfiguration;
import vn.hus.nlp.fsm.fst.FSTConfiguration;

public final class MinimalFSMBuilder extends FSMBuilder
{
  private Set<State> register = null;

  public MinimalFSMBuilder(String paramString)
  {
    super(paramString);
  }

  public Set<State> getRegister()
  {
    return this.register;
  }

  private Set<Character> getOutTransitionInputs(State paramState)
  {
    HashSet localHashSet = new HashSet();
    char[] arrayOfChar = paramState.getOutTransitionInputs();
    for (int i = 0; i < arrayOfChar.length; i++)
      localHashSet.add(new Character(arrayOfChar[i]));
    return localHashSet;
  }

  private boolean isEquivalent(State paramState1, State paramState2)
  {
    if ((paramState1 == null) || (paramState2 == null))
      return false;
    if ((paramState1.isFinalState()) && (!paramState2.isFinalState()))
      return false;
    if ((!paramState1.isFinalState()) && (paramState2.isFinalState()))
      return false;
    if (paramState1.equals(paramState2))
      return true;
    Set localSet1 = getOutTransitionInputs(paramState1);
    Set localSet2 = getOutTransitionInputs(paramState2);
    if (!localSet1.equals(localSet2))
      return false;
    Iterator localIterator = localSet1.iterator();
    while (localIterator.hasNext())
    {
      Character localCharacter = (Character)localIterator.next();
      State localState1 = this.machine.getNextState(paramState1, localCharacter.charValue());
      State localState2 = this.machine.getNextState(paramState2, localCharacter.charValue());
      if ((localState1 != null) && (!localState1.equals(localState2)))
        return false;
    }
    return true;
  }

  private void redirectTransitions(State paramState1, State paramState2)
  {
    Map localMap = this.machine.getIntransitionMap();
    List localList = (List)localMap.get(Integer.valueOf(paramState1.getId()));
    Iterator localIterator = localList.iterator();
    while (localIterator.hasNext())
    {
      Transition localTransition = (Transition)localIterator.next();
      if (localTransition.getTarget() == paramState1.getId())
        localTransition.setTarget(paramState2.getId());
    }
  }

  private State lastChild(State paramState)
  {
    List localList = paramState.getOutTransitions();
    int i = -1;
    Iterator localIterator = localList.iterator();
    while (localIterator.hasNext())
    {
      Transition localTransition = (Transition)localIterator.next();
      if (i < localTransition.getTarget())
        i = localTransition.getTarget();
    }
    if (i == paramState.getId())
    {
      System.err.println("Error: infinite loop!!! Id = " + i);
      printMachine();
      System.exit(1);
    }
    return this.machine.getState(i);
  }

  private void addSuffix(State paramState, String paramString)
  {
    for (int i = 0; i < paramString.length(); i++)
    {
      State localState = new State(this.machine.getStates().size());
      this.machine.addState(localState);
      char c = paramString.charAt(i);
      this.machine.addTransition(new Transition(paramState.getId(), localState.getId(), c));
      paramState = localState;
    }
    paramState.setType((byte)2);
  }

  private void addSuffix(State paramState, String paramString, String[] paramArrayOfString)
  {
    for (int i = 0; i < paramString.length(); i++)
    {
      State localState = new State(this.machine.getStates().size());
      this.machine.addState(localState);
      char c = paramString.charAt(i);
      this.machine.addTransition(new Transition(paramState.getId(), localState.getId(), c, paramArrayOfString[i]));
      paramState = localState;
    }
    paramState.setType((byte)2);
  }

  private void replaceOrRegister(State paramState)
  {
    State localState1 = lastChild(paramState);
    if (localState1 == null)
      return;
    if (localState1.getOutTransitions().size() > 0)
      replaceOrRegister(localState1);
    int i = 0;
    Iterator localIterator = this.register.iterator();
    while (localIterator.hasNext())
    {
      State localState2 = (State)localIterator.next();
      if (isEquivalent(localState2, localState1))
      {
        i = 1;
        redirectTransitions(localState1, localState2);
        this.machine.removeState(localState1);
        break;
      }
    }
    if (i == 0)
      this.register.add(localState1);
  }

  protected void addItem(String paramString, String[] paramArrayOfString)
  {
    paramString = paramString.trim();
    if (paramString.length() == 0)
      return;
    Configuration localConfiguration = getSimulator().track(paramString);
    State localState = localConfiguration.getCurrentState();
    String str = "";
    Object localObject;
    if ((localConfiguration instanceof DFAConfiguration))
    {
      localObject = (DFAConfiguration)localConfiguration;
      str = ((DFAConfiguration)localObject).getUnprocessedInput();
    }
    if ((localConfiguration instanceof FSTConfiguration))
    {
      localObject = (FSTConfiguration)localConfiguration;
      str = ((FSTConfiguration)localObject).getUnprocessedInput();
    }
    if (localState.getOutTransitions().size() > 0)
      replaceOrRegister(localState);
    if (paramArrayOfString == null)
    {
      addSuffix(localState, str);
    }
    else
    {
      int i = str.length();
      String[] arrayOfString = new String[i];
      int j = paramArrayOfString.length;
      for (int k = 0; k < i; k++)
        arrayOfString[k] = paramArrayOfString[(j - i + k)];
      addSuffix(localState, str, arrayOfString);
    }
  }

  public void create(String[] paramArrayOfString, String[][] paramArrayOfString1)
  {
    System.out.println("Building the minimal machine...");
    synchronized (this.machine)
    {
      long l1 = System.currentTimeMillis();
      for (int i = 0; i < paramArrayOfString.length; i++)
      {
        addItem(paramArrayOfString[i], paramArrayOfString1[i]);
        if (i % 1000 != 0)
          continue;
        System.out.println(" i = " + i);
      }
      finalize();
      long l2 = System.currentTimeMillis();
      long l3 = l2 - l1;
      System.out.println("Time to build the minimal machine = " + l3 + " (ms)");
    }
  }

  protected void finalize()
  {
    replaceOrRegister(this.machine.getInitialState());
  }

  public void dispose()
  {
    super.dispose();
    this.register.clear();
    this.register = null;
  }
}

/* Location:           C:\Downloads\vn.hus.nlp.tokenizer-4.1.1-bin.tar\lib\vn.hus.nlp.fsm-1.0.0.jar
 * Qualified Name:     vn.hus.nlp.fsm.builder.MinimalFSMBuilder
 * JD-Core Version:    0.6.0
 */